#
# Patch unrarsrc-3.1.3 to compile with DJGPP.
# After applying the patch issue the command
# make -f makefile.dj
# If you don't have UPX remove the upx command from makefile.dj and uncomment
# the strip command.
#
# NOTE: The patch to find.cpp fixes a bug where the file access mode of the
# archive to be extracted was set to read-only (DOS effect of chmod 0).
# attr=chmod(name,0) seemed to be intended to be written as attr=_chmod(name,0)
# (note the leading underscore), a DJGPP extension to simply get the file mode
# instead of using a quite expensive stat() call. But since we already did a
# stat() before we don't need to redo anything again..
#
# Sat Jan  4 22:32:53 CET 2003
# Eike Rathke <er@erack.de>
#
diff -u -N --exclude='*.o' --exclude='*.exe' unrar/filefn.cpp unrar.dj/filefn.cpp
--- unrar/filefn.cpp	2002-12-28 04:28:04.000000000 +0100
+++ unrar.dj/filefn.cpp	2003-01-04 17:43:34.000000000 +0100
@@ -168,6 +168,7 @@
   return(1457664);
 #elif defined(_EMX)
   int Drive=(!isalpha(FileName[0]) || FileName[1]!=':') ? 0:toupper(FileName[0])-'A'+1;
+#ifndef _DJGPP
   if (_osmode == OS2_MODE)
   {
     FSALLOCATE fsa;
@@ -178,12 +179,17 @@
     return(FreeSize);
   }
   else
+#endif
   {
     union REGS regs,outregs;
     memset(&regs,0,sizeof(regs));
     regs.h.ah=0x36;
     regs.h.dl=Drive;
+#ifdef _DJGPP
+    int86 (0x21,&regs,&outregs);
+#else
     _int86 (0x21,&regs,&outregs);
+#endif
     if (outregs.x.ax==0xffff)
       return(1457664);
     Int64 FreeSize=outregs.x.ax*outregs.x.cx;
diff -u -N --exclude='*.o' --exclude='*.exe' unrar/find.cpp unrar.dj/find.cpp
--- unrar/find.cpp	2002-12-28 04:28:04.000000000 +0100
+++ unrar.dj/find.cpp	2003-01-04 19:49:00.000000000 +0100
@@ -149,9 +149,7 @@
       fd->Error=(errno!=ENOENT);
       return(false);
     }
-#ifdef _DJGPP
-  fd->FileAttr=chmod(FindMask,0);
-#elif defined(_EMX)
+#if defined(_EMX) && !defined(_DJGPP)
   fd->FileAttr=st.st_attr;
 #else
   fd->FileAttr=st.st_mode;
diff -u -N --exclude='*.o' --exclude='*.exe' unrar/makefile.dj unrar.dj/makefile.dj
--- unrar/makefile.dj	1970-01-01 01:00:00.000000000 +0100
+++ unrar.dj/makefile.dj	2003-01-04 18:40:54.000000000 +0100
@@ -0,0 +1,43 @@
+#
+# Makefile for DOS DJGPP - unrar
+#
+# Note: you have to 'make clean' before you can build
+#	the sfx module
+#
+
+# DOS using DJGPP
+CXX=gpp
+CXXFLAGS=-O2
+DEFINES=-D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE -DUNRAR
+
+##########################
+
+COMPILE=$(CXX) $(CXXFLAGS) $(DEFINES)
+LINK=$(CXX)
+
+UNRAR_OBJ=filestr.o recvol.o rs.o scantree.o
+
+OBJECTS=rar.o strlist.o strfn.o pathfn.o int64.o savepos.o global.o file.o filefn.o filcreat.o \
+	archive.o arcread.o unicode.o system.o isnt.o crypt.o crc.o rawread.o encname.o \
+	resource.o match.o timefn.o rdwrfn.o consio.o options.o ulinks.o errhnd.o rarvm.o \
+	rijndael.o getbits.o sha1.o extinfo.o extract.o volume.o list.o find.o unpack.o cmddata.o
+
+.cpp.o:
+	$(COMPILE) -c $<
+
+all:	unrar.exe
+
+clean:
+	-rm *.o unrar.exe default.sfx
+
+unrar.exe:	$(OBJECTS) $(UNRAR_OBJ)
+	@rm -f makeunrar
+	$(LINK) -o unrar.exe $(LDFLAGS) $(OBJECTS) $(UNRAR_OBJ) $(LIBS)	
+#	@strip unrar.exe
+	upx unrar.exe
+
+sfx:	$(OBJECTS)
+	@rm -f default.sfx
+	$(LINK) -o default.sfx $(LDFLAGS) $(OBJECTS) -DSFX_MODULE 
+	strip default.sfx
+
diff -u -N --exclude='*.o' --exclude='*.exe' unrar/pathfn.cpp unrar.dj/pathfn.cpp
--- unrar/pathfn.cpp	2002-12-28 04:28:04.000000000 +0100
+++ unrar.dj/pathfn.cpp	2003-01-04 17:37:58.000000000 +0100
@@ -290,6 +290,7 @@
     strcpy(RARFileName,FullName);
     return;
   }
+#ifndef _DJGPP
   if (_osmode==OS2_MODE)
   {
     PTIB ptib;
@@ -298,6 +299,7 @@
     DosQueryModuleName(ppib->pib_hmte,NM,FullName);
   }
   else
+#endif
     strcpy(FullName,RARFileName);
   strcpy(PointToName(FullName),Name);
 #elif defined(_UNIX)
